@@ -8,36 +8,69 @@ _client = None
 project_id = None
 write_key = None
 read_key = None
+master_key = None
+base_url = None
 
 
 def _initialize_client_from_environment():
-    global _client, project_id, write_key, read_key
+    ''' Initialize a KeenCLient instance using environment variables. '''
+    global _client, project_id, write_key, read_key, master_key, base_url
 
     if _client is None:
         # check environment for project ID and keys
         project_id = project_id or os.environ.get("KEEN_PROJECT_ID")
         write_key = write_key or os.environ.get("KEEN_WRITE_KEY")
         read_key = read_key or os.environ.get("KEEN_READ_KEY")
+        master_key = master_key or os.environ.get("KEEN_MASTER_KEY")
+        base_url = base_url or os.environ.get("KEEN_BASE_URL")
 
         if not project_id:
             raise InvalidEnvironmentError("Please set the KEEN_PROJECT_ID environment variable or set keen.project_id!")
 
         _client = KeenClient(project_id,
                              write_key=write_key,
-                             read_key=read_key)
+                             read_key=read_key,
+                             master_key=master_key,
+                             base_url=base_url)
 
 
 def add_event(event_collection, body, timestamp=None):
+    """ Adds an event.
+
+    Depending on the persistence strategy of the client,
+    this will either result in the event being uploaded to Keen
+    immediately or will result in saving the event to some local cache.
+
+    :param event_collection: the name of the collection to insert the
+    event to
+    :param body: dict, the body of the event to insert the event to
+    :param timestamp: datetime, optional, the timestamp of the event
+    """
     _initialize_client_from_environment()
     _client.add_event(event_collection, body, timestamp=timestamp)
 
 
 def add_events(events):
+    """ Adds a batch of events.
+
+    Depending on the persistence strategy of the client,
+    this will either result in the event being uploaded to Keen
+    immediately or will result in saving the event to some local cache.
+
+    :param events: dictionary of events
+    """
     _initialize_client_from_environment()
     _client.add_events(events)
 
 
 def generate_image_beacon(event_collection, body, timestamp=None):
+    """ Generates an image beacon URL.
+
+    :param event_collection: the name of the collection to insert the
+    event to
+    :param body: dict, the body of the event to insert the event to
+    :param timestamp: datetime, optional, the timestamp of the event
+    """
     _initialize_client_from_environment()
     return _client.generate_image_beacon(event_collection, body, timestamp=timestamp)
 
@@ -55,7 +88,7 @@ def count(event_collection, timeframe=None, timezone=None, interval=None, filter
     :param interval: string, the time interval used for measuring data over
     time example: "daily"
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param group_by: string or array of strings, the name(s) of the properties you would
     like to group you results by.  example: "customer.id" or ["browser","operating_system"]
     :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
@@ -82,7 +115,7 @@ def sum(event_collection, target_property, timeframe=None, timezone=None, interv
     :param interval: string, the time interval used for measuring data over
     time example: "daily"
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param group_by: string or array of strings, the name(s) of the properties you would
     like to group you results by.  example: "customer.id" or ["browser","operating_system"]
     :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
@@ -110,7 +143,7 @@ def minimum(event_collection, target_property, timeframe=None, timezone=None, in
     :param interval: string, the time interval used for measuring data over
     time example: "daily"
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param group_by: string or array of strings, the name(s) of the properties you would
     like to group you results by.  example: "customer.id" or ["browser","operating_system"]
     :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
@@ -138,7 +171,7 @@ def maximum(event_collection, target_property, timeframe=None, timezone=None, in
     :param interval: string, the time interval used for measuring data over
     time example: "daily"
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param group_by: string or array of strings, the name(s) of the properties you would
     like to group you results by.  example: "customer.id" or ["browser","operating_system"]
     :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
@@ -166,7 +199,7 @@ def average(event_collection, target_property, timeframe=None, timezone=None, in
     :param interval: string, the time interval used for measuring data over
     time example: "daily"
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param group_by: string or array of strings, the name(s) of the properties you would
     like to group you results by.  example: "customer.id" or ["browser","operating_system"]
     :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
@@ -179,6 +212,34 @@ def average(event_collection, target_property, timeframe=None, timezone=None, in
                            target_property=target_property, max_age=max_age)
 
 
+def median(event_collection, target_property, timeframe=None, timezone=None, interval=None, filters=None,
+           group_by=None, max_age=None):
+    """ Performs a median query
+
+    Finds the median of a target property for events that meet the given criteria.
+
+    :param event_collection: string, the name of the collection to query
+    :param target_property: string, the name of the event property you would like use
+    :param timeframe: string or dict, the timeframe in which the events
+    happened example: "previous_7_days"
+    :param timezone: int, the timezone you'd like to use for the timeframe
+    and interval in seconds
+    :param interval: string, the time interval used for measuring data over
+    time example: "daily"
+    :param filters: array of dict, contains the filters you'd like to apply to the data
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    :param group_by: string or array of strings, the name(s) of the properties you would
+    like to group you results by.  example: "customer.id" or ["browser","operating_system"]
+    :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
+    willing to trade for increased query performance, in seconds
+
+    """
+    _initialize_client_from_environment()
+    return _client.median(event_collection=event_collection, timeframe=timeframe, timezone=timezone,
+                          interval=interval, filters=filters, group_by=group_by,
+                          target_property=target_property, max_age=max_age)
+
+
 def percentile(event_collection, target_property, percentile, timeframe=None, timezone=None, interval=None,
                filters=None, group_by=None, max_age=None):
     """ Performs a percentile query
@@ -196,7 +257,7 @@ def percentile(event_collection, target_property, percentile, timeframe=None, ti
     :param interval: string, the time interval used for measuring data over
     time example: "daily"
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param group_by: string or array of strings, the name(s) of the properties you would
     like to group you results by.  example: "customer.id" or ["browser","operating_system"]
     :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
@@ -232,7 +293,7 @@ def count_unique(event_collection, target_property, timeframe=None, timezone=Non
     :param interval: string, the time interval used for measuring data over
     time example: "daily"
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param group_by: string or array of strings, the name(s) of the properties you would
     like to group you results by.  example: "customer.id" or ["browser","operating_system"]
     :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
@@ -260,7 +321,7 @@ def select_unique(event_collection, target_property, timeframe=None, timezone=No
     :param interval: string, the time interval used for measuring data over
     time example: "daily"
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param group_by: string or array of strings, the name(s) of the properties you would
     like to group you results by.  example: "customer.id" or ["browser","operating_system"]
     :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
@@ -286,7 +347,7 @@ def extraction(event_collection, timeframe=None, timezone=None, filters=None, la
     :param timezone: int, the timezone you'd like to use for the timeframe
     and interval in seconds
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param latest: int, the number of most recent records you'd like to return
     :param email: string, optional string containing an email address to email results to
     :param property_names: string or list of strings, used to limit the properties returned
@@ -297,7 +358,7 @@ def extraction(event_collection, timeframe=None, timezone=None, filters=None, la
                               filters=filters, latest=latest, email=email, property_names=property_names)
 
 
-def funnel(steps, timeframe=None, timezone=None, max_age=None):
+def funnel(*args, **kwargs):
     """ Performs a Funnel query
 
     Returns an object containing the results for each step of the funnel.
@@ -314,7 +375,7 @@ def funnel(steps, timeframe=None, timezone=None, max_age=None):
 
     """
     _initialize_client_from_environment()
-    return _client.funnel(steps=steps, timeframe=timeframe, timezone=timezone, max_age=max_age)
+    return _client.funnel(*args, **kwargs)
 
 
 def multi_analysis(event_collection, analyses, timeframe=None, interval=None,
@@ -334,7 +395,7 @@ def multi_analysis(event_collection, analyses, timeframe=None, interval=None,
     :param timezone: int, the timezone you'd like to use for the timeframe
     and interval in seconds
     :param filters: array of dict, contains the filters you'd like to apply to the data
-    example: {["property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
     :param group_by: string or array of strings, the name(s) of the properties you would
     like to group you results by.  example: "customer.id" or ["browser","operating_system"]
     :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
@@ -345,3 +406,39 @@ def multi_analysis(event_collection, analyses, timeframe=None, interval=None,
     return _client.multi_analysis(event_collection=event_collection, timeframe=timeframe,
                                   interval=interval, timezone=timezone, filters=filters,
                                   group_by=group_by, analyses=analyses, max_age=max_age)
+
+
+def delete_events(*args, **kwargs):
+    """ Performs a delete for events.
+
+    Returns true upon success.
+
+    :param event_collection: string, the event collection from which event are being deleted
+    :param timeframe: string or dict, the timeframe in which the events
+    happened example: "previous_7_days"
+    :param timezone: int, the timezone you'd like to use for the timeframe
+    and interval in seconds
+    :param filters: array of dict, contains the filters you'd like to apply to the data
+    example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+
+    """
+    _initialize_client_from_environment()
+    return _client.delete_events(*args, **kwargs)
+
+
+def get_collection(*args, **kwargs):
+    """ Returns event collection schema
+
+    :param event_collection: string, the event collection from which schema is to be returned,
+    if left blank will return schema for all collections
+    """
+    _initialize_client_from_environment()
+    return _client.get_collection(*args, **kwargs)
+
+
+def get_all_collections():
+    """ Returns event collection schema for all events
+
+    """
+    _initialize_client_from_environment()
+    return _client.get_all_collections()
