@@ -2,7 +2,7 @@ import base64
 import copy
 import json
 import sys
-from keen import persistence_strategies, exceptions
+from keen import persistence_strategies, exceptions, saved_queries
 from keen.api import KeenApi
 from keen.persistence_strategies import BasePersistenceStrategy
 
@@ -37,7 +37,10 @@ class Event(object):
         """
         event_as_dict = copy.deepcopy(self.event_body)
         if self.timestamp:
-            event_as_dict["keen"] = {"timestamp": self.timestamp.isoformat()}
+            if "keen" in event_as_dict:
+                event_as_dict["keen"]["timestamp"] = self.timestamp.isoformat()
+            else:
+                event_as_dict["keen"] = {"timestamp": self.timestamp.isoformat()}
         return json.dumps(event_as_dict)
 
 
@@ -57,7 +60,8 @@ class KeenClient(object):
     """
 
     def __init__(self, project_id, write_key=None, read_key=None,
-                 persistence_strategy=None, api_class=KeenApi, get_timeout=305, post_timeout=305):
+                 persistence_strategy=None, api_class=KeenApi, get_timeout=305, post_timeout=305,
+                 master_key=None, base_url=None):
         """ Initializes a KeenClient object.
 
         :param project_id: the Keen IO project ID
@@ -67,6 +71,7 @@ class KeenClient(object):
         the event
         :param get_timeout: optional, the timeout on GET requests
         :param post_timeout: optional, the timeout on POST requests
+        :param master_key: a Keen IO Master API Key
         """
         super(KeenClient, self).__init__()
 
@@ -76,7 +81,8 @@ class KeenClient(object):
         # Set up an api client to be used for querying and optionally passed
         # into a default persistence strategy.
         self.api = api_class(project_id, write_key=write_key, read_key=read_key,
-                             get_timeout=get_timeout, post_timeout=post_timeout)
+                             get_timeout=get_timeout, post_timeout=post_timeout,
+                             master_key=master_key, base_url=base_url)
 
         if persistence_strategy:
             # validate the given persistence strategy
@@ -91,6 +97,7 @@ class KeenClient(object):
         self.persistence_strategy = persistence_strategy
         self.get_timeout = get_timeout
         self.post_timeout = post_timeout
+        self.saved_queries = saved_queries.SavedQueriesInterface(project_id, master_key, read_key)
 
     if sys.version_info[0] < 3:
         @staticmethod
@@ -152,6 +159,37 @@ class KeenClient(object):
             self.api.write_key.decode(sys.getdefaultencoding()), self._base64_encode(event_json)
         )
 
+    def delete_events(self, event_collection, timeframe=None, timezone=None, filters=None):
+        """ Deletes events.
+
+        :param event_collection: string, the event collection from which event are being deleted
+        :param timeframe: string or dict, the timeframe in which the events happened
+        example: "previous_7_days"
+        :param timezone: int, the timezone you'd like to use for the timeframe
+        and interval in seconds
+        :param filters: array of dict, contains the filters you'd like to apply to the data
+        example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+
+        """
+        params = self.get_params(timeframe=timeframe, timezone=timezone, filters=filters)
+        return self.api.delete_events(event_collection, params)
+
+    def get_collection(self, event_collection):
+        """ Returns event collection schema
+
+        :param event_collection: string, the event collection from which schema is to be returned,
+        if left blank will return schema for all collections
+        """
+
+        return self.api.get_collection(event_collection)
+
+    def get_all_collections(self):
+        """ Returns event collection schema for all events
+
+        """
+
+        return self.api.get_all_collections()
+
     def _base64_encode(self, string_to_encode):
         """ Base64 encodes a string, with either Python 2 or 3.
 
@@ -307,6 +345,33 @@ class KeenClient(object):
                                  target_property=target_property, max_age=max_age)
         return self.api.query("average", params)
 
+    def median(self, event_collection, target_property, timeframe=None, timezone=None, interval=None,
+                filters=None, group_by=None, max_age=None):
+        """ Performs a median query
+
+        Finds the median of a target property for events that meet the given criteria.
+
+        :param event_collection: string, the name of the collection to query
+        :param target_property: string, the name of the event property you would like use
+        :param timeframe: string or dict, the timeframe in which the events
+        happened example: "previous_7_days"
+        :param timezone: int, the timezone you'd like to use for the timeframe
+        and interval in seconds
+        :param interval: string, the time interval used for measuring data over
+        time example: "daily"
+        :param filters: array of dict, contains the filters you'd like to apply to the data
+        example: [{"property_name":"device", "operator":"eq", "property_value":"iPhone"}]
+        :param group_by: string or array of strings, the name(s) of the properties you would
+        like to group you results by.  example: "customer.id" or ["browser","operating_system"]
+        :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
+        willing to trade for increased query performance, in seconds
+
+        """
+        params = self.get_params(event_collection=event_collection, timeframe=timeframe, timezone=timezone,
+                                 interval=interval, filters=filters, group_by=group_by,
+                                 target_property=target_property, max_age=max_age)
+        return self.api.query("median", params)
+
     def percentile(self, event_collection, target_property, percentile, timeframe=None, timezone=None,
                    interval=None, filters=None, group_by=None, max_age=None):
         """ Performs a percentile query
@@ -421,7 +486,7 @@ class KeenClient(object):
                                  filters=filters, latest=latest, email=email, property_names=property_names)
         return self.api.query("extraction", params)
 
-    def funnel(self, steps, timeframe=None, timezone=None, max_age=None):
+    def funnel(self, steps, timeframe=None, timezone=None, max_age=None, all_keys=False):
         """ Performs a Funnel query
 
         Returns an object containing the results for each step of the funnel.
@@ -435,10 +500,17 @@ class KeenClient(object):
         and interval in seconds
         :param max_age: an integer, greater than 30 seconds, the maximum 'staleness' you're
         willing to trade for increased query performance, in seconds
+        :all_keys: set to true to return all keys on response (i.e. "result", "actors", "steps")
 
         """
-        params = self.get_params(steps=steps, timeframe=timeframe, timezone=timezone, max_age=max_age)
-        return self.api.query("funnel", params)
+        params = self.get_params(
+            steps=steps,
+            timeframe=timeframe,
+            timezone=timezone,
+            max_age=max_age,
+        )
+
+        return self.api.query("funnel", params, all_keys=all_keys)
 
     def multi_analysis(self, event_collection, analyses, timeframe=None, interval=None, timezone=None, filters=None,
                        group_by=None, max_age=None):
